import os

HTTP_PROTOCOL = "http"
HTTPS_PROTOCOL = "https"
LOCALHOST = os.environ.get("LOCALHOST") or '127.0.0.1'

USE_DOCKER_NETWORK = os.environ.get("USE_DOCKER_NETWORK") == 'True' and True
DEFAULT_DOCKER_PORT = 8000




class _NoneDataType:
    def __init__(self):
        pass

    def __bool__(self):
        return False

    def __repr__(self):
        return "NoneData"

    def __getattr__(self, item):
        return self


NoneData = _NoneDataType()

__all__ = ['NoneData']



from ib_core_lite.settings import HTTPS_PROTOCOL, HTTP_PROTOCOL, LOCALHOST, USE_DOCKER_NETWORK, DEFAULT_DOCKER_PORT


def check_status_code(status_code: int) -> bool:
    """
    Проеверка статус-кода запроса на успех
    :param status_code: int
    :return: True если запрос прошел успешно, иначе False
    """
    if 2 <= status_code / 100 < 3:
        return True
    return False


def create_root_url(use_ssl: bool = False, domain: str = LOCALHOST, port: int | None = None) -> str:
    """
    Генерация корневого URL (протокол + доменное имя)
    :param use_ssl: Использовать ssl (протокол https)
    :param domain: Доменное имя
    :param port: Порт (используется в случае запроса на localhost)
    :return: сгенерированный корневой URL
    """
    return f"{HTTPS_PROTOCOL if use_ssl else HTTP_PROTOCOL}://{domain}{f':{port}' if port is not None else ''}"


def generate_url(
        use_ssl=False,
        domain=LOCALHOST,
        port: int = None,
        use_ending_slash=False,
        *args: str, container_name: str = "", **kwargs: str):
    """
    Генерация URL
    :param use_ssl: Использовать ssl (протокол https).
    :param domain: Доменное имя.
    :param port: Порт (используется в случае запроса на localhost).
    :param use_ending_slash: Поставить слеш ('/') в конце URL
    :param args: параметры для генерации URL. Пример: *args("api", "object", "create") -> "api/object/create"
    :param kwargs: GET параметры запроса. Пример: *kwargs{"query": "example"} -> "?query=example"
    :param container_name
    :return: Сгенерированный URL
    """
    url = create_root_url(use_ssl, domain, port) \
        if not USE_DOCKER_NETWORK or not container_name \
        else create_root_url(use_ssl, container_name, DEFAULT_DOCKER_PORT)
    for arg in args:
        url += f"/{arg}"
    if use_ending_slash:
        url += "/"
    if kwargs.keys():
        url += "?"
        for key in kwargs.keys():
            url += f"{key}={kwargs[key]}"
            url += "&"
        url = url[:-1]
    return url



import typing

import requests

from ib_core_lite.settings import LOCALHOST
from ib_core_lite.utils import generate_url

auth_types = typing.Literal[
    "Token",
    "Bearer"
]


def token_authorization(_token, auth_type: auth_types):
    url = generate_url(True, "ib-elp.psg.su", None, False, "ib_auth", "authorization",
                       "token" if auth_type == "Token" else "jwt")
    auth = requests.get(
        url=url,
        headers={
            "Authorization": f"{auth_type} {_token}"
        }
    )
    if int(auth.status_code / 100) >= 5:
        return {"message": "Authorization server returned fatal error"}, auth.status_code
    return auth.json(), auth.status_code


def get_auth_header(request):
    return request.headers.get("Authorization", "")


def get_auth_info(authorization: str):
    if not authorization:
        auth_type, token = "", ""
    else:
        try:
            auth_type, token = tuple(authorization.split(' ', 1))
        except Exception as ex:  # noqa
            auth_type, token = "", ""
    return auth_type, token


def check_group_permission(auth_content: str, group_data):
    response = requests.post(
        url=generate_url(False, LOCALHOST, 8001, False, "user", "group", "operations", "check", "list"),
        json={"groups": group_data},
        headers={
            "Authorization": auth_content
        }
    )
    print(response.json())
    try:
        return response.json()["has_permission"]
    except:  # noqa
        return False



from ipaddress import ip_network, ip_address

from fastapi.responses import JSONResponse
from starlette import status
from starlette.requests import Request

from ib_core_lite.authorization.types import *
from ib_core_lite.authorization.utils import get_auth_info, token_authorization, check_group_permission


class Token:
    """
    Token model for API authorization
    """

    def __init__(self, _type: token_types):
        """
        Token class constructor.

        Keyword arguments:
            _type (token_types): token type. Supports BaseToken and JsonWebToken (from ib_core.tokens.types)
        """
        self._type = _type

    """
    NOT OVERRIDER
    """
    __API_AUTHORIZATION_TYPES = {
        None: EMPTY,
        BASE_TOKEN: BASE_TOKEN_AUTH,
        JWT: JWT_AUTH
    }

    @property
    def type(self):
        return self._type

    @property
    def auth_type(self):
        return self.__API_AUTHORIZATION_TYPES[self._type]

    def __str__(self):
        return str(self._type)


class AuthUser:
    """
    User model for external authorization
    """

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)

    def get_username(self):
        return getattr(self, 'username')

    def get_id(self):
        return getattr(self, 'id')

    def is_superuser(self):
        return getattr(self, 'is_superuser')


class AuthorizationReader:
    """
    Request authorization header reader class
    """
    authorization = None
    auth_type = None
    token = None
    user_id = None

    def __init__(self, request, user_id: str = None):
        self.user_id = user_id
        self._get_authorization(request)

    def _get_authorization(self, request):
        self.authorization = request.headers.get("Authorization", "")
        self.auth_type, self.token = get_auth_info(self.authorization)

    def get_authorization_header_content(self):
        return f"{self.auth_type} {self.token}" if not self.user_id else self.user_id


class TokenAuthorizationMixin:
    """
    Uses external authorization of the IB-ELP Auth module.

    Usage:
        1) add mixin to view;
        2) define the types of tokens

    """
    user = None
    user_id = None
    tokens: list[Token] = [Token(BASE_TOKEN), Token(JWT)]
    auth = None
    permission_groups: typing.List[str] = []
    only_admins = False
    message = None
    status = None
    market_id = None
    net1 = ip_network("10.0.0.0/8")
    net2 = ip_network("100.64.0.0/10")
    net3 = ip_network("172.16.0.0/12")
    net4 = ip_network("192.168.0.0/16")
    net5 = ip_network("127.0.0.0/8")
    DEFAULT_USER_ID = "00000000-0000-0000-0000-000000000000"
    USER_ID_HEADER = "X-UserID"

    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)
        self.dispatch(self.get_property("request"))

    def get_property(self, key):
        return getattr(self, key)

    def generate_exception(self):
        print(self.status, self.message)
        return JSONResponse(status_code=self.status, content={"detail": self.message})

    def permission_groups_to_data(self):
        return [{"name": group} for group in self.permission_groups]

    def check_token(self):
        if not self.tokens or not isinstance(self.tokens, list):
            raise Exception("'tokens' should be 'Token's objects list")

    def get_allowed_authorization_types(self):
        return [_token.auth_type for _token in self.tokens]

    def set_message_and_status(self, message: str, _status: int):
        self.message = message
        self.status = _status

    @classmethod
    def is_local_address(cls, request):
        ADDRESS = request.client.host
        print(ADDRESS)
        return ip_address(ADDRESS) in cls.net1 or ip_address(ADDRESS) in cls.net2 or ip_address(
            ADDRESS) in cls.net3 or ip_address(
            ADDRESS) in cls.net4 or ip_address(
            ADDRESS) in cls.net5

    def get_market(self, request):
        self.market_id = request.headers.get("market")

    def dispatch(self, request: Request, *args, **kwargs):
        self.get_market(request)
        if self.is_local_address(request):
            self.user_id = request.headers.get(self.USER_ID_HEADER)
            if self.user_id:
                self.auth = AuthorizationReader(request, self.user_id)
                self.set_message_and_status(message="Авторизован", _status=200)
                return
        self.check_token()
        allowed_authorization_types = self.get_allowed_authorization_types()
        self.auth = AuthorizationReader(request)
        if not self.auth.authorization:
            self.message = "Учетные данные не были предоставлены."
            self.status = status.HTTP_401_UNAUTHORIZED
            return

        if self.auth.auth_type not in allowed_authorization_types:
            self.message = "Некорректный тип авторизации."
            self.status = status.HTTP_401_UNAUTHORIZED
            return

        response, _status = token_authorization(
            self.auth.token,
            self.auth.auth_type
        )

        if not (2 <= int(_status / 100) < 3):
            self.set_message_and_status(response.get("detail"), _status)
            return

        self.user = AuthUser(**response)
        self.user_id = self.user.get_id()
        self.auth.user_id = self.user_id
        if self.only_admins and not self.user.is_superuser:
            self.message = "Доступ запрещен."
            self.status = status.HTTP_403_FORBIDDEN
            return
        self.set_message_and_status(message="Авторизован", _status=200)
        return



import typing

EMPTY = ""
BASE_TOKEN_AUTH = "Token"
JWT_AUTH = "Bearer"

auth_types = typing.Literal[
    "Token",
    "Bearer"
]

NONE = ""
BASE_TOKEN = "BaseToken"
JWT = "JWT"

token_types = typing.Literal[
    "BaseToken",
    "JWT"
]



from typing import TypeVar
import requests

from ib_core_lite.types import NoneData
from ib_core_lite.utils import check_status_code, generate_url
from ib_core_lite.settings import LOCALHOST

T = TypeVar('T')
TPort = TypeVar('TPort', int, None)
THeaders = TypeVar('THeaders', dict, None)
TData = TypeVar('TData', dict, None)


class METHODS:
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"
    OPTIONS = "OPTIONS"
    HEAD = "HEAD"


class ResponseData:
    def __init__(self, **attributes):
        for key, value in attributes.items():
            if isinstance(value, dict):
                setattr(self, key, ResponseData(**value))
            elif isinstance(value, list):
                setattr(self, key, [ResponseData(**item) if isinstance(item, dict) else item for item in value])
            else:
                setattr(self, key, value)

    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            return NoneData

    def to_dict(self) -> dict:
        obj_dict = {}
        for key, value in self.__dict__.items():
            if isinstance(value, ResponseData):
                obj_dict[key] = value.to_dict()
            elif isinstance(value, list):
                obj_dict[key] = [item.to_dict() if isinstance(item, Response) else item for item in value]
            else:
                obj_dict[key] = value
        return obj_dict

    def __str__(self):
        return str(self.to_dict())


class Response:
    status: int
    data: ResponseData
    success: bool

    def __init__(self, status: int, **data):
        self.status = status
        self.data = ResponseData(**data)
        self.success = check_status_code(status)


class Request:
    METHODS = METHODS

    @classmethod
    def _resp_has_json(cls, status) -> bool:
        return 2 <= status / 100 < 3 or 4 <= status / 100 < 5

    @classmethod
    def request(
            cls,
            *url_args: str,
            method: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        url = generate_url(
            not local, LOCALHOST if local else domain, port, use_ending_slash,
            *url_args, container_name=container_name, **url_kwargs
        )
        response = requests.request(
            method=method,
            url=url,
            headers=headers,
            json=data
        )
        data, status = response.json() if cls._resp_has_json(response.status_code) else dict(), response.status_code
        return Response(status, **data)

    @classmethod
    def get(
            cls,
            *url_args: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        return cls.request(
            *url_args,
            method=METHODS.GET,
            local=local,
            port=port,
            use_ending_slash=use_ending_slash,
            domain=domain,
            container_name=container_name,
            headers=headers,
            data=data,
            **url_kwargs
        )

    @classmethod
    def post(
            cls,
            *url_args: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        return cls.request(
            *url_args,
            method=METHODS.POST,
            local=local,
            port=port,
            use_ending_slash=use_ending_slash,
            domain=domain,
            container_name=container_name,
            headers=headers,
            data=data,
            **url_kwargs
        )

    @classmethod
    def put(
            cls,
            *url_args: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        return cls.request(
            *url_args,
            method=METHODS.PUT,
            local=local,
            port=port,
            use_ending_slash=use_ending_slash,
            domain=domain,
            container_name=container_name,
            headers=headers,
            data=data,
            **url_kwargs
        )

    @classmethod
    def patch(
            cls,
            *url_args: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        return cls.request(
            *url_args,
            method=METHODS.PATCH,
            local=local,
            port=port,
            use_ending_slash=use_ending_slash,
            domain=domain,
            container_name=container_name,
            headers=headers,
            data=data,
            **url_kwargs
        )

    @classmethod
    def delete(
            cls,
            *url_args: str,
            local: bool = True,
            port: TPort = None,
            use_ending_slash: bool = False,
            domain: str = "",
            container_name: str = "",
            headers: THeaders = None,
            data: TData = None,
            **url_kwargs
    ):
        return cls.request(
            *url_args,
            method=METHODS.DELETE,
            local=local,
            port=port,
            use_ending_slash=use_ending_slash,
            domain=domain,
            container_name=container_name,
            headers=headers,
            data=data,
            **url_kwargs
        )



try:
    from setuptools import setup, find_packages
except ImportError:
    raise ImportError('Please install setuptools.')

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='ib_core_lite',
    version='1.0.0',
    author='Damir Fatkhullin',
    author_email='damir.f@it-psg.com',
    url='https://gitlab.it-psg.com/ib-elp-it-psg/ib_core_lite',
    description='Integration Bus fastapi-applications core',
    download_url='https://gitlab.it-psg.com/ib-elp-it-psg/ib_core_lite.git',
    license='MIT',
    packages=find_packages(),
    package_data={
        'ib_core_lite': ['ib_core_lite/*'],
    },
    install_requires=['fastapi', 'starlette', 'requests'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
)